@@ -1,9 +1,16 @@
-
 class Error(Exception):
     """Base exception class for all ufo2ft errors."""
+
     pass
 
 
 class InvalidFontData(Error):
     """Raised when input font contains invalid data."""
+
+    pass
+
+
+class InvalidDesignSpaceData(Error):
+    """Raised when input DesignSpace document contains invalid data."""
+
     pass
