@@ -1,9 +1,4 @@
-from __future__ import (
-    print_function,
-    division,
-    absolute_import,
-    unicode_literals,
-)
+from __future__ import print_function, division, absolute_import, unicode_literals
 from fontTools.misc.py23 import unichr, basestring, SimpleNamespace
 from fontTools.misc.fixedTools import otRound
 
@@ -99,6 +94,12 @@ DIST_ENABLED_SCRIPTS = {
     "Gonm",  # Masaram Gondi
     "Soyo",  # Soyombo
     "Zanb",  # Zanabazar Square
+    # Unicode-11.0 additions
+    "Dogr",  # Dogra
+    "Gong",  # Gunjala Gondi
+    "Maka",  # Makasar
+    # Unicode-12.0 additions
+    "Nand",  # Nandinagari
 }
 
 
@@ -114,21 +115,20 @@ def unicodeScriptDirection(uv):
     return unicodedata.script_horizontal_direction(sc)
 
 
-STRONG_LTR_BIDI_TYPE = "L"
-STRONG_RTL_BIDI_TYPES = {"R", "AL"}
-LTR_NUMBER_BIDI_TYPES = {"AN", "EN"}
+RTL_BIDI_TYPES = {"R", "AL"}
+LTR_BIDI_TYPES = {"L", "AN", "EN"}
 
 
 def unicodeBidiType(uv):
-    # return "R", "L", "N" (for numbers), or None for everything else
+    """Return "R" for characters with RTL direction, or "L" for LTR (whether
+    'strong' or 'weak'), or None for neutral direction.
+    """
     char = unichr(uv)
     bidiType = unicodedata.bidirectional(char)
-    if bidiType in STRONG_RTL_BIDI_TYPES:
+    if bidiType in RTL_BIDI_TYPES:
         return "R"
-    elif bidiType == STRONG_LTR_BIDI_TYPE:
+    elif bidiType in LTR_BIDI_TYPES:
         return "L"
-    elif bidiType in LTR_NUMBER_BIDI_TYPES:
-        return "N"
     else:
         return None
 
@@ -207,7 +207,8 @@ class KernFeatureWriter(BaseFeatureWriter):
         ctx = super(KernFeatureWriter, self).setContext(
             font, feaFile, compiler=compiler
         )
-        ctx.kerning = self.getKerningData(font, feaFile)
+        ctx.gdefClasses = ast.getGDEFGlyphClasses(feaFile)
+        ctx.kerning = self.getKerningData(font, feaFile, self.getOrderedGlyphSet())
 
         feaScripts = ast.getScriptLanguageSystems(feaFile)
         ctx.scriptGroups = self._groupScriptsByTagAndDirection(feaScripts)
@@ -219,10 +220,7 @@ class KernFeatureWriter(BaseFeatureWriter):
             self.log.debug("No kerning data; skipped")
             return False
 
-        if (
-            "dist" in self.context.todo
-            and "dist" not in self.context.scriptGroups
-        ):
+        if "dist" in self.context.todo and "dist" not in self.context.scriptGroups:
             self.log.debug(
                 "No dist-enabled scripts defined in languagesystem "
                 "statements; dist feature will not be generated"
@@ -256,8 +254,8 @@ class KernFeatureWriter(BaseFeatureWriter):
             statements.append(ast.Comment(""))
 
         # finally add the lookup and feature blocks
-        for _, lookup in sorted(lookups.items()):
-            statements.append(lookup)
+        for _, lookupGroup in sorted(lookups.items()):
+            statements.extend(lookupGroup)
         if "kern" in features:
             statements.append(features["kern"])
         if "dist" in features:
@@ -265,20 +263,23 @@ class KernFeatureWriter(BaseFeatureWriter):
         return True
 
     @classmethod
-    def getKerningData(cls, font, feaFile=None):
-        side1Classes, side2Classes = cls.getKerningClasses(font, feaFile)
-        pairs = cls.getKerningPairs(font, side1Classes, side2Classes)
+    def getKerningData(cls, font, feaFile=None, glyphSet=None):
+        side1Classes, side2Classes = cls.getKerningClasses(font, feaFile, glyphSet)
+        pairs = cls.getKerningPairs(font, side1Classes, side2Classes, glyphSet)
         return SimpleNamespace(
             side1Classes=side1Classes, side2Classes=side2Classes, pairs=pairs
         )
 
     @staticmethod
-    def getKerningGroups(font):
-        allGlyphs = set(font.keys())
+    def getKerningGroups(font, glyphSet=None):
+        if glyphSet:
+            allGlyphs = set(glyphSet.keys())
+        else:
+            allGlyphs = set(font.keys())
         side1Groups = {}
         side2Groups = {}
         for name, members in font.groups.items():
-            # prune non-existent glyphs
+            # prune non-existent or skipped glyphs
             members = [g for g in members if g in allGlyphs]
             if not members:
                 # skip empty groups
@@ -291,8 +292,8 @@ class KernFeatureWriter(BaseFeatureWriter):
         return side1Groups, side2Groups
 
     @classmethod
-    def getKerningClasses(cls, font, feaFile=None):
-        side1Groups, side2Groups = cls.getKerningGroups(font)
+    def getKerningClasses(cls, font, feaFile=None, glyphSet=None):
+        side1Groups, side2Groups = cls.getKerningGroups(font, glyphSet)
         side1Classes = ast.makeGlyphClassDefinitions(
             side1Groups, feaFile, stripPrefix="public."
         )
@@ -302,8 +303,11 @@ class KernFeatureWriter(BaseFeatureWriter):
         return side1Classes, side2Classes
 
     @staticmethod
-    def getKerningPairs(font, side1Classes, side2Classes):
-        allGlyphs = set(font.keys())
+    def getKerningPairs(font, side1Classes, side2Classes, glyphSet=None):
+        if glyphSet:
+            allGlyphs = set(glyphSet.keys())
+        else:
+            allGlyphs = set(font.keys())
         kerning = font.kerning
 
         pairsByFlags = {}
@@ -362,7 +366,7 @@ class KernFeatureWriter(BaseFeatureWriter):
     def _makePairPosRule(pair, rtl=False):
         enumerated = pair.firstIsClass ^ pair.secondIsClass
         value = otRound(pair.value)
-        if rtl and "N" in pair.bidiTypes:
+        if rtl and "L" in pair.bidiTypes:
             # numbers are always shaped LTR even in RTL scripts
             rtl = False
         valuerecord = ast.ValueRecord(
@@ -379,19 +383,19 @@ class KernFeatureWriter(BaseFeatureWriter):
             enumerated=enumerated,
         )
 
-    def _makeKerningLookup(self, name, pairs, exclude=None, rtl=False):
+    def _makeKerningLookup(
+        self, name, pairs, exclude=None, rtl=False, ignoreMarks=True
+    ):
         assert pairs
         rules = []
         for pair in pairs:
             if exclude is not None and exclude(pair):
-                self.log.debug(
-                    "pair excluded from '%s' lookup: %r", name, pair
-                )
+                self.log.debug("pair excluded from '%s' lookup: %r", name, pair)
                 continue
             rules.append(self._makePairPosRule(pair, rtl=rtl))
         if rules:
             lookup = ast.LookupBlock(name)
-            if self.options.ignoreMarks:
+            if ignoreMarks and self.options.ignoreMarks:
                 lookup.statements.append(ast.makeLookupFlag("IgnoreMarks"))
             lookup.statements.extend(rules)
             return lookup
@@ -414,14 +418,20 @@ class KernFeatureWriter(BaseFeatureWriter):
         else:
             shouldSplit = False
 
+        marks = self.context.gdefClasses.mark
         lookups = {}
         if shouldSplit:
             # make one DFLT lookup with script-agnostic characters, and two
             # LTR/RTL lookups excluding pairs from the opposite group.
-            # We drop kerning pairs with ambiguous direction.
+            # We drop kerning pairs with ambiguous direction: i.e. those containing
+            # glyphs from scripts with different overall horizontal direction, or
+            # glyphs with incompatible bidirectional type (e.g. arabic letters vs
+            # arabic numerals).
             pairs = []
             for pair in self.context.kerning.pairs:
-                if "RTL" in pair.directions and "LTR" in pair.directions:
+                if ("RTL" in pair.directions and "LTR" in pair.directions) or (
+                    "R" in pair.bidiTypes and "L" in pair.bidiTypes
+                ):
                     self.log.warning(
                         "skipped kern pair with ambiguous direction: %r", pair
                     )
@@ -430,47 +440,79 @@ class KernFeatureWriter(BaseFeatureWriter):
             if not pairs:
                 return lookups
 
-            dfltKern = self._makeKerningLookup(
-                "kern_dflt",
-                pairs,
-                exclude=(
-                    lambda pair: {"LTR", "RTL"}.intersection(pair.directions)
-                ),
-                rtl=False,
-            )
-            if dfltKern:
-                lookups["DFLT"] = dfltKern
-
-            ltrKern = self._makeKerningLookup(
-                "kern_ltr",
-                pairs,
-                exclude=(
-                    lambda pair: not pair.directions
-                    or "RTL" in pair.directions
-                ),
-                rtl=False,
-            )
-            if ltrKern:
-                lookups["LTR"] = ltrKern
-
-            rtlKern = self._makeKerningLookup(
-                "kern_rtl",
-                pairs,
-                exclude=(
-                    lambda pair: not pair.directions
-                    or "LTR" in pair.directions
-                ),
-                rtl=True,
-            )
-            if rtlKern:
-                lookups["RTL"] = rtlKern
+            if self.options.ignoreMarks:
+                # If there are pairs with a mix of mark/base then the IgnoreMarks
+                # flag is unnecessary and should not be set
+                basePairs, markPairs = self._splitBaseAndMarkPairs(pairs, marks)
+                self._makeSplitDirectionKernLookups(lookups, basePairs)
+                if markPairs:
+                    self._makeSplitDirectionKernLookups(
+                        lookups, markPairs, ignoreMarks=False, suffix="_marks"
+                    )
+            else:
+                self._makeSplitDirectionKernLookups(lookups, pairs)
         else:
-            # only make a single (implicitly LTR) lookup including all pairs
-            lookups["LTR"] = self._makeKerningLookup(
-                "kern_ltr", self.context.kerning.pairs
-            )
+            # only make a single (implicitly LTR) lookup including all base/base pairs
+            # and a single lookup including all base/mark pairs (if any)
+            pairs = self.context.kerning.pairs
+            if self.options.ignoreMarks:
+                basePairs, markPairs = self._splitBaseAndMarkPairs(pairs, marks)
+                lookups["LTR"] = [self._makeKerningLookup("kern_ltr", basePairs)]
+                if markPairs:
+                    lookups["LTR"].append(
+                        self._makeKerningLookup(
+                            "kern_ltr_marks", markPairs, ignoreMarks=False
+                        )
+                    )
+            else:
+                lookups["LTR"] = [self._makeKerningLookup("kern_ltr", pairs)]
         return lookups
 
+    def _splitBaseAndMarkPairs(self, pairs, marks):
+        basePairs, markPairs = [], []
+        if marks:
+            for pair in pairs:
+                if any(glyph in marks for glyph in pair.glyphs):
+                    markPairs.append(pair)
+                else:
+                    basePairs.append(pair)
+        else:
+            basePairs[:] = pairs
+        return basePairs, markPairs
+
+    def _makeSplitDirectionKernLookups(
+        self, lookups, pairs, ignoreMarks=True, suffix=""
+    ):
+        dfltKern = self._makeKerningLookup(
+            "kern_dflt" + suffix,
+            pairs,
+            exclude=(lambda pair: {"LTR", "RTL"}.intersection(pair.directions)),
+            rtl=False,
+            ignoreMarks=ignoreMarks,
+        )
+        if dfltKern:
+            lookups.setdefault("DFLT", []).append(dfltKern)
+
+        ltrKern = self._makeKerningLookup(
+            "kern_ltr" + suffix,
+            pairs,
+            exclude=(lambda pair: not pair.directions or "RTL" in pair.directions),
+            rtl=False,
+            ignoreMarks=ignoreMarks,
+        )
+        if ltrKern:
+            lookups.setdefault("LTR", []).append(ltrKern)
+
+        rtlKern = self._makeKerningLookup(
+            "kern_rtl" + suffix,
+            pairs,
+            exclude=(lambda pair: not pair.directions or "LTR" in pair.directions),
+            rtl=True,
+            ignoreMarks=ignoreMarks,
+        )
+        if rtlKern:
+            lookups.setdefault("RTL", []).append(rtlKern)
+
     def _makeFeatureBlocks(self, lookups):
         features = {}
         if "kern" in self.context.todo:
@@ -487,7 +529,8 @@ class KernFeatureWriter(BaseFeatureWriter):
 
     def _registerKernLookups(self, feature, lookups):
         if "DFLT" in lookups:
-            ast.addLookupReference(feature, lookups["DFLT"])
+            ast.addLookupReferences(feature, lookups["DFLT"])
+
         scriptGroups = self.context.scriptGroups
         if "dist" in self.context.todo:
             distScripts = scriptGroups["dist"]
@@ -496,22 +539,23 @@ class KernFeatureWriter(BaseFeatureWriter):
         kernScripts = scriptGroups.get("kern", {})
         ltrScripts = kernScripts.get("LTR", [])
         rtlScripts = kernScripts.get("RTL", [])
-        ltrLkp = lookups.get("LTR")
-        rtlLkp = lookups.get("RTL")
-        if ltrLkp and rtlLkp:
+
+        ltrLookups = lookups.get("LTR")
+        rtlLookups = lookups.get("RTL")
+        if ltrLookups and rtlLookups:
             if ltrScripts and rtlScripts:
                 for script, langs in ltrScripts:
-                    ast.addLookupReference(feature, ltrLkp, script, langs)
+                    ast.addLookupReferences(feature, ltrLookups, script, langs)
                 for script, langs in rtlScripts:
-                    ast.addLookupReference(feature, rtlLkp, script, langs)
+                    ast.addLookupReferences(feature, rtlLookups, script, langs)
             elif ltrScripts:
-                ast.addLookupReference(feature, rtlLkp, script="DFLT")
+                ast.addLookupReferences(feature, rtlLookups, script="DFLT")
                 for script, langs in ltrScripts:
-                    ast.addLookupReference(feature, ltrLkp, script, langs)
+                    ast.addLookupReferences(feature, ltrLookups, script, langs)
             elif rtlScripts:
-                ast.addLookupReference(feature, ltrLkp, script="DFLT")
+                ast.addLookupReferences(feature, ltrLookups, script="DFLT")
                 for script, langs in rtlScripts:
-                    ast.addLookupReference(feature, rtlLkp, script, langs)
+                    ast.addLookupReferences(feature, rtlLookups, script, langs)
             else:
                 if not (distScripts.get("LTR") and distScripts.get("RTL")):
                     raise ValueError(
@@ -519,28 +563,28 @@ class KernFeatureWriter(BaseFeatureWriter):
                         "lookups; add 'languagesystems' to features for at "
                         "least one LTR or RTL script using the kern feature"
                     )
-        elif ltrLkp:
+        elif ltrLookups:
             if not (rtlScripts or distScripts):
-                ast.addLookupReference(feature, ltrLkp)
+                ast.addLookupReferences(feature, ltrLookups)
             else:
-                ast.addLookupReference(feature, ltrLkp, script="DFLT")
+                ast.addLookupReferences(feature, ltrLookups, script="DFLT")
                 for script, langs in ltrScripts:
-                    ast.addLookupReference(feature, ltrLkp, script, langs)
-        elif rtlLkp:
+                    ast.addLookupReferences(feature, ltrLookups, script, langs)
+        elif rtlLookups:
             if not (ltrScripts or distScripts):
-                ast.addLookupReference(feature, rtlLkp)
+                ast.addLookupReferences(feature, rtlLookups)
             else:
-                ast.addLookupReference(feature, rtlLkp, script="DFLT")
+                ast.addLookupReferences(feature, rtlLookups, script="DFLT")
                 for script, langs in rtlScripts:
-                    ast.addLookupReference(feature, rtlLkp, script, langs)
+                    ast.addLookupReferences(feature, rtlLookups, script, langs)
 
     def _registerDistLookups(self, feature, lookups):
         scripts = self.context.scriptGroups["dist"]
-        ltrLkp = lookups.get("LTR")
-        if ltrLkp:
+        ltrLookups = lookups.get("LTR")
+        if ltrLookups:
             for script, langs in scripts.get("LTR", []):
-                ast.addLookupReference(feature, ltrLkp, script, langs)
-        rtlLkp = lookups.get("RTL")
-        if rtlLkp:
+                ast.addLookupReferences(feature, ltrLookups, script, langs)
+        rtlLookups = lookups.get("RTL")
+        if rtlLookups:
             for script, langs in scripts.get("RTL", []):
-                ast.addLookupReference(feature, rtlLkp, script, langs)
+                ast.addLookupReferences(feature, rtlLookups, script, langs)
