@@ -36,6 +36,27 @@ logger = logging.getLogger(__name__)
 # generic
 _styleMapStyleNames = ["regular", "bold", "italic", "bold italic"]
 
+
+def ascenderFallback(info):
+    upm = getAttrWithFallback(info, "unitsPerEm")
+    return otRound(upm * 0.8)
+
+
+def descenderFallback(info):
+    upm = getAttrWithFallback(info, "unitsPerEm")
+    return -otRound(upm * 0.2)
+
+
+def capHeightFallback(info):
+    upm = getAttrWithFallback(info, "unitsPerEm")
+    return otRound(upm * 0.7)
+
+
+def xHeightFallback(info):
+    upm = getAttrWithFallback(info, "unitsPerEm")
+    return otRound(upm * 0.5)
+
+
 def styleMapFamilyNameFallback(info):
     """
     Fallback to *openTypeNamePreferredFamilyName* if
@@ -54,6 +75,7 @@ def styleMapFamilyNameFallback(info):
         styleName = ""
     return (familyName + " " + styleName).strip()
 
+
 def styleMapStyleNameFallback(info):
     """
     Fallback to *openTypeNamePreferredSubfamilyName* if
@@ -69,13 +91,16 @@ def styleMapStyleNameFallback(info):
         styleName = styleName.strip().lower()
     return styleName
 
+
 # head
 
 _date_format = "%Y/%m/%d %H:%M:%S"
 
+
 def dateStringForNow():
     return time.strftime(_date_format, time.gmtime())
 
+
 def openTypeHeadCreatedFallback(info):
     """
     Fallback to the environment variable SOURCE_DATE_EPOCH if set, otherwise
@@ -87,19 +112,25 @@ def openTypeHeadCreatedFallback(info):
     else:
         return dateStringForNow()
 
+
 # hhea
 
+
 def openTypeHheaAscenderFallback(info):
     """
     Fallback to *ascender + typoLineGap*.
     """
-    return info.ascender + getAttrWithFallback(info, "openTypeOS2TypoLineGap")
+    return getAttrWithFallback(info, "ascender") + getAttrWithFallback(
+        info, "openTypeOS2TypoLineGap"
+    )
+
 
 def openTypeHheaDescenderFallback(info):
     """
     Fallback to *descender*.
     """
-    return info.descender
+    return getAttrWithFallback(info, "descender")
+
 
 def openTypeHheaCaretSlopeRiseFallback(info):
     """
@@ -110,14 +141,17 @@ def openTypeHheaCaretSlopeRiseFallback(info):
     """
     italicAngle = getAttrWithFallback(info, "italicAngle")
     if italicAngle != 0:
-        if (hasattr(info, "openTypeHheaCaretSlopeRun") and
-                info.openTypeHheaCaretSlopeRun is not None):
+        if (
+            hasattr(info, "openTypeHheaCaretSlopeRun")
+            and info.openTypeHheaCaretSlopeRun is not None
+        ):
             slopeRun = info.openTypeHheaCaretSlopeRun
             return otRound(slopeRun / math.tan(math.radians(-italicAngle)))
         else:
             return 1000  # just an arbitrary non-zero reference point
     return 1
 
+
 def openTypeHheaCaretSlopeRunFallback(info):
     """
     Fallback to *openTypeHheaCaretSlopeRun*. If the italicAngle is zero,
@@ -130,8 +164,10 @@ def openTypeHheaCaretSlopeRunFallback(info):
         return otRound(math.tan(math.radians(-italicAngle)) * slopeRise)
     return 0
 
+
 # name
 
+
 def openTypeNameVersionFallback(info):
     """
     Fallback to *versionMajor.versionMinor* in the form 0.000.
@@ -140,6 +176,7 @@ def openTypeNameVersionFallback(info):
     versionMinor = getAttrWithFallback(info, "versionMinor")
     return "Version %d.%s" % (versionMajor, str(versionMinor).zfill(3))
 
+
 def openTypeNameUniqueIDFallback(info):
     """
     Fallback to *openTypeNameVersion;openTypeOS2VendorID;postscriptFontName*.
@@ -149,17 +186,20 @@ def openTypeNameUniqueIDFallback(info):
     fontName = getAttrWithFallback(info, "postscriptFontName")
     return "%s;%s;%s" % (version, vendor, fontName)
 
+
 def openTypeNamePreferredFamilyNameFallback(info):
     """
     Fallback to *familyName*.
     """
-    return info.familyName
+    return getAttrWithFallback(info, "familyName")
+
 
 def openTypeNamePreferredSubfamilyNameFallback(info):
     """
     Fallback to *styleName*.
     """
-    return info.styleName
+    return getAttrWithFallback(info, "styleName")
+
 
 def openTypeNameCompatibleFullNameFallback(info):
     """
@@ -173,51 +213,68 @@ def openTypeNameCompatibleFullNameFallback(info):
         familyName += " " + styleMapStyleName.title()
     return familyName
 
+
 def openTypeNameWWSFamilyNameFallback(info):
     # not yet supported
     return None
 
+
 def openTypeNameWWSSubfamilyNameFallback(info):
     # not yet supported
     return None
 
+
 # OS/2
 
+
 def openTypeOS2TypoAscenderFallback(info):
     """
     Fallback to *ascender*.
     """
-    return info.ascender
+    return getAttrWithFallback(info, "ascender")
+
 
 def openTypeOS2TypoDescenderFallback(info):
     """
     Fallback to *descender*.
     """
-    return info.descender
+    return getAttrWithFallback(info, "descender")
+
 
 def openTypeOS2TypoLineGapFallback(info):
     """
     Fallback to *UPM * 1.2 - ascender + descender*, or zero if that's negative.
     """
-    return max(int(info.unitsPerEm * 1.2) - info.ascender + info.descender, 0)
+    return max(
+        int(getAttrWithFallback(info, "unitsPerEm") * 1.2)
+        - getAttrWithFallback(info, "ascender")
+        + getAttrWithFallback(info, "descender"),
+        0,
+    )
+
 
 def openTypeOS2WinAscentFallback(info):
     """
     Fallback to *ascender + typoLineGap*.
     """
-    return info.ascender + getAttrWithFallback(info, "openTypeOS2TypoLineGap")
+    return getAttrWithFallback(info, "ascender") + getAttrWithFallback(
+        info, "openTypeOS2TypoLineGap"
+    )
+
 
 def openTypeOS2WinDescentFallback(info):
     """
     Fallback to *descender*.
     """
-    return abs(info.descender)
+    return abs(getAttrWithFallback(info, "descender"))
+
 
 # postscript
 
 _postscriptFontNameExceptions = set("[](){}<>/%")
 _postscriptFontNameAllowed = set([unichr(i) for i in range(33, 127)])
 
+
 def normalizeStringForPostscript(s, allowSpaces=True):
     s = tounicode(s)
     normalized = []
@@ -234,23 +291,33 @@ def normalizeStringForPostscript(s, allowSpaces=True):
         normalized.append(tostr(c))
     return "".join(normalized)
 
+
 def normalizeNameForPostscript(name):
     return normalizeStringForPostscript(name, allowSpaces=False)
 
+
 def postscriptFontNameFallback(info):
     """
     Fallback to a string containing only valid characters
     as defined in the specification. This will draw from
     *openTypeNamePreferredFamilyName* and *openTypeNamePreferredSubfamilyName*.
     """
-    name = "%s-%s" % (getAttrWithFallback(info, "openTypeNamePreferredFamilyName"), getAttrWithFallback(info, "openTypeNamePreferredSubfamilyName"))
+    name = "%s-%s" % (
+        getAttrWithFallback(info, "openTypeNamePreferredFamilyName"),
+        getAttrWithFallback(info, "openTypeNamePreferredSubfamilyName"),
+    )
     return normalizeNameForPostscript(name)
 
+
 def postscriptFullNameFallback(info):
     """
     Fallback to *openTypeNamePreferredFamilyName openTypeNamePreferredSubfamilyName*.
     """
-    return "%s %s" % (getAttrWithFallback(info, "openTypeNamePreferredFamilyName"), getAttrWithFallback(info, "openTypeNamePreferredSubfamilyName"))
+    return "%s %s" % (
+        getAttrWithFallback(info, "openTypeNamePreferredFamilyName"),
+        getAttrWithFallback(info, "openTypeNamePreferredSubfamilyName"),
+    )
+
 
 def postscriptSlantAngleFallback(info):
     """
@@ -258,17 +325,18 @@ def postscriptSlantAngleFallback(info):
     """
     return getAttrWithFallback(info, "italicAngle")
 
+
 def postscriptUnderlineThicknessFallback(info):
     """Return UPM * 0.05 (50 for 1000 UPM) and warn."""
-    logger.warning(
-        'Underline thickness not set in UFO, defaulting to UPM * 0.05')
-    return info.unitsPerEm * 0.05
+    logger.debug("Underline thickness not set in UFO, defaulting to UPM * 0.05")
+    return getAttrWithFallback(info, "unitsPerEm") * 0.05
+
 
 def postscriptUnderlinePositionFallback(info):
     """Return UPM * -0.075 (-75 for 1000 UPM) and warn."""
-    logger.warning(
-        'Underline position not set in UFO, defaulting to UPM * -0.075')
-    return info.unitsPerEm * -0.075
+    logger.debug("Underline position not set in UFO, defaulting to UPM * -0.075")
+    return getAttrWithFallback(info, "unitsPerEm") * -0.075
+
 
 def postscriptBlueScaleFallback(info):
     """
@@ -283,15 +351,16 @@ def postscriptBlueScaleFallback(info):
     if blues:
         assert len(blues) % 2 == 0
         for x, y in zip(blues[:-1:2], blues[1::2]):
-            maxZoneHeight = max(maxZoneHeight, abs(y-x))
+            maxZoneHeight = max(maxZoneHeight, abs(y - x))
     if otherBlues:
         assert len(otherBlues) % 2 == 0
         for x, y in zip(otherBlues[:-1:2], otherBlues[1::2]):
-            maxZoneHeight = max(maxZoneHeight, abs(y-x))
+            maxZoneHeight = max(maxZoneHeight, abs(y - x))
     if maxZoneHeight != 0:
-        blueScale = 3/(4*maxZoneHeight)
+        blueScale = 3 / (4 * maxZoneHeight)
     return blueScale
 
+
 # --------------
 # Attribute Maps
 # --------------
@@ -301,17 +370,17 @@ staticFallbackData = dict(
     versionMinor=0,
     copyright=None,
     trademark=None,
+    familyName="New Font",
+    styleName="Regular",
+    unitsPerEm=1000,
     italicAngle=0,
     # not needed
     year=None,
     note=None,
-
     openTypeHeadLowestRecPPEM=6,
     openTypeHeadFlags=[0, 1],
-
     openTypeHheaLineGap=0,
     openTypeHheaCaretOffset=0,
-
     openTypeNameDesigner=None,
     openTypeNameDesignerURL=None,
     openTypeNameManufacturer=None,
@@ -321,7 +390,6 @@ staticFallbackData = dict(
     openTypeNameDescription=None,
     openTypeNameSampleText=None,
     openTypeNameRecords=[],
-
     openTypeOS2WidthClass=5,
     openTypeOS2WeightClass=400,
     openTypeOS2Selection=[],
@@ -331,7 +399,6 @@ staticFallbackData = dict(
     openTypeOS2UnicodeRanges=None,
     openTypeOS2CodePageRanges=None,
     openTypeOS2Type=[2],
-
     openTypeOS2SubscriptXSize=None,
     openTypeOS2SubscriptYSize=None,
     openTypeOS2SubscriptXOffset=None,
@@ -342,7 +409,6 @@ staticFallbackData = dict(
     openTypeOS2SuperscriptYOffset=None,
     openTypeOS2StrikeoutSize=None,
     openTypeOS2StrikeoutPosition=None,
-
     # fallback to None on these
     # as the user should be in
     # complete control
@@ -355,7 +421,6 @@ staticFallbackData = dict(
     openTypeVheaCaretSlopeRise=0,
     openTypeVheaCaretSlopeRun=1,
     openTypeVheaCaretOffset=0,
-
     postscriptUniqueID=None,
     postscriptWeightName=None,
     postscriptIsFixedPitch=False,
@@ -370,17 +435,19 @@ staticFallbackData = dict(
     postscriptForceBold=0,
     postscriptDefaultWidthX=200,
     postscriptNominalWidthX=0,
-
     # not used in OTF
     postscriptDefaultCharacter=None,
     postscriptWindowsCharacterSet=None,
-
     # not used in OTF
     macintoshFONDFamilyID=None,
-    macintoshFONDName=None
+    macintoshFONDName=None,
 )
 
 specialFallbacks = dict(
+    ascender=ascenderFallback,
+    descender=descenderFallback,
+    capHeight=capHeightFallback,
+    xHeight=xHeightFallback,
     styleMapFamilyName=styleMapFamilyNameFallback,
     styleMapStyleName=styleMapStyleNameFallback,
     openTypeHeadCreated=openTypeHeadCreatedFallback,
@@ -405,47 +472,52 @@ specialFallbacks = dict(
     postscriptSlantAngle=postscriptSlantAngleFallback,
     postscriptUnderlineThickness=postscriptUnderlineThicknessFallback,
     postscriptUnderlinePosition=postscriptUnderlinePositionFallback,
-    postscriptBlueScale=postscriptBlueScaleFallback
+    postscriptBlueScale=postscriptBlueScaleFallback,
 )
 
-requiredAttributes = set(ufoLib.fontInfoAttributesVersion2) - (set(staticFallbackData.keys()) | set(specialFallbacks.keys()))
+requiredAttributes = set(ufoLib.fontInfoAttributesVersion2) - (
+    set(staticFallbackData.keys()) | set(specialFallbacks.keys())
+)
 
-recommendedAttributes = set([
-    "styleMapFamilyName",
-    "versionMajor",
-    "versionMinor",
-    "copyright",
-    "trademark",
-    "openTypeHeadCreated",
-    "openTypeNameDesigner",
-    "openTypeNameDesignerURL",
-    "openTypeNameManufacturer",
-    "openTypeNameManufacturerURL",
-    "openTypeNameLicense",
-    "openTypeNameLicenseURL",
-    "openTypeNameDescription",
-    "openTypeNameSampleText",
-    "openTypeOS2WidthClass",
-    "openTypeOS2WeightClass",
-    "openTypeOS2VendorID",
-    "openTypeOS2Panose",
-    "openTypeOS2FamilyClass",
-    "openTypeOS2UnicodeRanges",
-    "openTypeOS2CodePageRanges",
-    "openTypeOS2TypoLineGap",
-    "openTypeOS2Type",
-    "postscriptBlueValues",
-    "postscriptOtherBlues",
-    "postscriptFamilyBlues",
-    "postscriptFamilyOtherBlues",
-    "postscriptStemSnapH",
-    "postscriptStemSnapV"
-])
+recommendedAttributes = set(
+    [
+        "styleMapFamilyName",
+        "versionMajor",
+        "versionMinor",
+        "copyright",
+        "trademark",
+        "openTypeHeadCreated",
+        "openTypeNameDesigner",
+        "openTypeNameDesignerURL",
+        "openTypeNameManufacturer",
+        "openTypeNameManufacturerURL",
+        "openTypeNameLicense",
+        "openTypeNameLicenseURL",
+        "openTypeNameDescription",
+        "openTypeNameSampleText",
+        "openTypeOS2WidthClass",
+        "openTypeOS2WeightClass",
+        "openTypeOS2VendorID",
+        "openTypeOS2Panose",
+        "openTypeOS2FamilyClass",
+        "openTypeOS2UnicodeRanges",
+        "openTypeOS2CodePageRanges",
+        "openTypeOS2TypoLineGap",
+        "openTypeOS2Type",
+        "postscriptBlueValues",
+        "postscriptOtherBlues",
+        "postscriptFamilyBlues",
+        "postscriptFamilyOtherBlues",
+        "postscriptStemSnapH",
+        "postscriptStemSnapV",
+    ]
+)
 
 # ------------
 # Main Methods
 # ------------
 
+
 def getAttrWithFallback(info, attr):
     """
     Get the value for *attr* from the *info* object.
@@ -463,6 +535,7 @@ def getAttrWithFallback(info, attr):
             value = staticFallbackData[attr]
     return value
 
+
 def preflightInfo(info):
     """
     Returns a dict containing two items. The value for each
@@ -483,16 +556,18 @@ def preflightInfo(info):
             missingRecommended.add(attr)
     return dict(missingRequired=missingRequired, missingRecommended=missingRecommended)
 
+
 # -----------------
 # Low Level Support
 # -----------------
 
 # these should not be used outside of this package
 
+
 def intListToNum(intList, start, length):
     all = []
     bin = ""
-    for i in range(start, start+length):
+    for i in range(start, start + length):
         if i in intList:
             b = "1"
         else:
@@ -507,6 +582,7 @@ def intListToNum(intList, start, length):
     all = " ".join(all)
     return binary2num(all)
 
+
 def dateStringToTimeValue(date):
     try:
         t = time.strptime(date, "%Y/%m/%d %H:%M:%S")
