@@ -26,6 +26,7 @@ from fontTools.ttLib.tables._g_l_y_f import Glyph, USE_MY_METRICS
 from fontTools.misc.arrayTools import unionRect
 from fontTools.misc.fixedTools import otRound
 
+from ufo2ft.errors import InvalidFontData
 from ufo2ft.fontInfoData import (
     getAttrWithFallback,
     dateStringToTimeValue,
@@ -38,6 +39,7 @@ from ufo2ft.util import (
     makeUnicodeToGlyphNameMapping,
     calcCodePageRanges,
 )
+from ufo2ft.constants import COLOR_LAYERS_KEY, COLOR_PALETTES_KEY
 
 
 logger = logging.getLogger(__name__)
@@ -69,7 +71,8 @@ class BaseOutlineCompiler(object):
 
     sfntVersion = None
     tables = frozenset(
-        ["head", "hmtx", "hhea", "name", "maxp", "cmap", "OS/2", "post", "vmtx", "vhea"]
+        ["head", "hmtx", "hhea", "name", "maxp", "cmap", "OS/2", "post", "vmtx", "vhea",
+         "COLR", "CPAL"]
     )
 
     def __init__(self, font, glyphSet=None, glyphOrder=None, tables=None):
@@ -108,6 +111,8 @@ class BaseOutlineCompiler(object):
             getAttrWithFallback(self.ufo.info, metric) is not None
             for metric in vertical_metrics
         )
+        self.colorLayers = COLOR_LAYERS_KEY in self.ufo.lib and \
+                           COLOR_PALETTES_KEY in self.ufo.lib
 
         # write the glyph order
         self.otf.setGlyphOrder(self.glyphOrder)
@@ -124,6 +129,9 @@ class BaseOutlineCompiler(object):
         if self.vertical:
             self.setupTable_vmtx()
             self.setupTable_vhea()
+        if self.colorLayers:
+            self.setupTable_COLR()
+            self.setupTable_CPAL()
         self.setupOtherTables()
         self.importTTX()
 
@@ -873,6 +881,42 @@ class BaseOutlineCompiler(object):
         post.minMemType1 = 0
         post.maxMemType1 = 0
 
+    def setupTable_COLR(self):
+        """
+        Compile the COLR table.
+
+        **This should not be called externally.**
+        """
+        if "COLR" not in self.tables:
+            return
+
+        from fontTools.colorLib.builder import buildCOLR
+
+        layerInfo = self.ufo.lib[COLOR_LAYERS_KEY]
+        self.otf["COLR"] = buildCOLR(layerInfo)
+
+    def setupTable_CPAL(self):
+        """
+        Compile the CPAL table.
+
+        **This should not be called externally.**
+        """
+        if "CPAL" not in self.tables:
+            return
+
+        from fontTools.colorLib.builder import buildCPAL
+        from fontTools.colorLib.errors import ColorLibError
+
+        # colorLib wants colors as tuples, plistlib gives us lists
+        palettes = [
+            [tuple(color) for color in palette]
+            for palette in self.ufo.lib[COLOR_PALETTES_KEY]
+        ]
+        try:
+            self.otf["CPAL"] = buildCPAL(palettes)
+        except ColorLibError as e:
+            raise InvalidFontData("Failed to build CPAL table") from e
+
     def setupOtherTables(self):
         """
         Make the other tables. The default implementation does nothing.
