@@ -1,7 +1,7 @@
-from __future__ import (
-    print_function, division, absolute_import, unicode_literals)
+from __future__ import print_function, division, absolute_import, unicode_literals
 
 from fontTools.misc.py23 import basestring
+from ufo2ft.fontInfoData import getAttrWithFallback
 from ufo2ft.filters import loadFilters
 from ufo2ft.filters.decomposeComponents import DecomposeComponentsFilter
 from ufo2ft.util import _GlyphSet
@@ -28,11 +28,15 @@ class BasePreProcessor(object):
     "com.github.googlei18n.ufo2ft.filters".
     """
 
-    def __init__(self, ufo, inplace=False, layerName=None, **kwargs):
+    def __init__(
+        self, ufo, inplace=False, layerName=None, skipExportGlyphs=None, **kwargs
+    ):
         self.ufo = ufo
         self.inplace = inplace
         self.layerName = layerName
-        self.glyphSet = _GlyphSet.from_layer(ufo, layerName, copy=not inplace)
+        self.glyphSet = _GlyphSet.from_layer(
+            ufo, layerName, copy=not inplace, skipExportGlyphs=skipExportGlyphs
+        )
         self.defaultFilters = self.initDefaultFilters(**kwargs)
         self.preFilters, self.postFilters = loadFilters(ufo)
 
@@ -60,17 +64,13 @@ class OTFPreProcessor(BasePreProcessor):
     ``RemoveOverlapsFilter.SKIA_PATHOPS``, or the string "pathops".
     """
 
-    def initDefaultFilters(
-        self, removeOverlaps=False, overlapsBackend=None
-    ):
+    def initDefaultFilters(self, removeOverlaps=False, overlapsBackend=None):
         filters = [DecomposeComponentsFilter()]
         if removeOverlaps:
             from ufo2ft.filters.removeOverlaps import RemoveOverlapsFilter
 
             if overlapsBackend is not None:
-                filters.append(
-                    RemoveOverlapsFilter(backend=overlapsBackend)
-                )
+                filters.append(RemoveOverlapsFilter(backend=overlapsBackend))
             else:
                 filters.append(RemoveOverlapsFilter())
 
@@ -118,7 +118,7 @@ class TTFPreProcessor(OTFPreProcessor):
         convertCubics=True,
         conversionError=None,
         reverseDirection=True,
-        rememberCurveType=True
+        rememberCurveType=True,
     ):
         # len(g) is the number of contours, so we include the all glyphs
         # that have both components and at least one contour
@@ -128,9 +128,7 @@ class TTFPreProcessor(OTFPreProcessor):
             from ufo2ft.filters.removeOverlaps import RemoveOverlapsFilter
 
             if overlapsBackend is not None:
-                filters.append(
-                    RemoveOverlapsFilter(backend=overlapsBackend)
-                )
+                filters.append(RemoveOverlapsFilter(backend=overlapsBackend))
             else:
                 filters.append(RemoveOverlapsFilter())
 
@@ -154,17 +152,31 @@ class TTFInterpolatablePreProcessor(object):
     The constructor takes a list of UFO fonts, and the ``process`` method
     returns the modified glyphsets (list of dicts) in the same order.
 
-    Currently, only the conversion from cubic to quadratic, and the
-    decomposition of mixed contour/component glyphs is performed,
-    and no additional custom filter is applied.
+    The pre-processor performs the conversion from cubic to quadratic on
+    all the UFOs at once, then decomposes mixed contour/component glyphs.
+
+    Additional pre/post custom filter are also applied to each single UFOs,
+    respectively before or after the default filters, if they are specified
+    in the UFO's lib.plist under the private key
+    "com.github.googlei18n.ufo2ft.filters".
+    NOTE: If you use any custom filters, the resulting glyphsets may no longer
+    be interpolation compatible, depending on the particular filter used or
+    whether they are applied to only some vs all of the UFOs.
 
     The ``conversionError``, ``reverseDirection`` and ``rememberCurveType``
     arguments work in the same way as in the ``TTFPreProcessor``.
     """
 
-    def __init__(self, ufos, inplace=False, conversionError=None,
-                 reverseDirection=True, rememberCurveType=True,
-                 layerNames=None):
+    def __init__(
+        self,
+        ufos,
+        inplace=False,
+        conversionError=None,
+        reverseDirection=True,
+        rememberCurveType=True,
+        layerNames=None,
+        skipExportGlyphs=None,
+    ):
         from cu2qu.ufo import DEFAULT_MAX_ERR
 
         self.ufos = ufos
@@ -176,18 +188,33 @@ class TTFInterpolatablePreProcessor(object):
         self.layerNames = layerNames
 
         self.glyphSets = [
-            _GlyphSet.from_layer(ufo, layerName, copy=not inplace)
+            _GlyphSet.from_layer(
+                ufo, layerName, copy=not inplace, skipExportGlyphs=skipExportGlyphs
+            )
             for ufo, layerName in zip(ufos, layerNames)
         ]
         self._conversionErrors = [
-            (conversionError or DEFAULT_MAX_ERR) * ufo.info.unitsPerEm
-            for ufo in ufos]
+            (conversionError or DEFAULT_MAX_ERR)
+            * getAttrWithFallback(ufo.info, "unitsPerEm")
+            for ufo in ufos
+        ]
         self._reverseDirection = reverseDirection
         self._rememberCurveType = rememberCurveType
 
+        self.preFilters, self.postFilters = [], []
+        for ufo in ufos:
+            pre, post = loadFilters(ufo)
+            self.preFilters.append(pre)
+            self.postFilters.append(post)
+
     def process(self):
         from cu2qu.ufo import fonts_to_quadratic
 
+        # first apply all custom pre-filters
+        for funcs, ufo, glyphSet in zip(self.preFilters, self.ufos, self.glyphSets):
+            for func in funcs:
+                func(ufo, glyphSet)
+
         fonts_to_quadratic(
             self.glyphSets,
             max_err=self._conversionErrors,
@@ -200,4 +227,9 @@ class TTFInterpolatablePreProcessor(object):
         for ufo, glyphSet in zip(self.ufos, self.glyphSets):
             decompose(ufo, glyphSet)
 
+        # finally apply all custom post-filters
+        for funcs, ufo, glyphSet in zip(self.postFilters, self.ufos, self.glyphSets):
+            for func in funcs:
+                func(ufo, glyphSet)
+
         return self.glyphSets
