@@ -1,10 +1,6 @@
 # -*- coding: utf-8 -*-
-from __future__ import (
-    print_function,
-    division,
-    absolute_import,
-    unicode_literals,
-)
+from __future__ import print_function, division, absolute_import, unicode_literals
+
 try:
     from inspect import getfullargspec as getargspec  # PY3
 except ImportError:
@@ -15,6 +11,9 @@ from fontTools import ttLib
 from fontTools import subset
 from fontTools import unicodedata
 from fontTools.feaLib.builder import addOpenTypeFeatures
+from fontTools.misc.transform import Identity, Transform
+from fontTools.pens.reverseContourPen import ReverseContourPen
+from fontTools.pens.transformPen import TransformPen
 import logging
 
 
@@ -47,19 +46,46 @@ def makeOfficialGlyphOrder(font, glyphOrder=None):
 
 
 class _GlyphSet(dict):
-
     @classmethod
-    def from_layer(cls, font, layerName=None, copy=False):
+    def from_layer(cls, font, layerName=None, copy=False, skipExportGlyphs=None):
+        """Return a mapping of glyph names to glyph objects from `font`."""
         if layerName is not None:
             layer = font.layers[layerName]
         else:
             layer = font.layers.defaultLayer
+
         if copy:
             self = _copyLayer(layer, obj_type=cls)
             self.lib = deepcopy(layer.lib)
         else:
             self = cls((g.name, g) for g in layer)
             self.lib = layer.lib
+
+        # If any glyphs in the skipExportGlyphs list are used as components, decompose
+        # them in the containing glyphs...
+        if skipExportGlyphs:
+            for glyph in self.values():
+                if any(c.baseGlyph in skipExportGlyphs for c in glyph.components):
+                    deepCopyContours(self, glyph, glyph, Transform(), skipExportGlyphs)
+                    if hasattr(glyph, "removeComponent"):  # defcon
+                        for c in [
+                            component
+                            for component in glyph.components
+                            if component.baseGlyph in skipExportGlyphs
+                        ]:
+                            glyph.removeComponent(c)
+                    else:  # ufoLib2
+                        glyph.components[:] = [
+                            c
+                            for c in glyph.components
+                            if c.baseGlyph not in skipExportGlyphs
+                        ]
+            # ... and then remove them from the glyph set, if even present.
+            for glyph_name in skipExportGlyphs:
+                if glyph_name in self:
+                    del self[glyph_name]
+
+        self.name = layer.name if layerName is not None else None
         return self
 
 
@@ -79,9 +105,11 @@ def _copyLayer(layer, obj_type=dict):
     else:
 
         def newGlyph(name):
-            g = cls()
-            g.name = name
-            return g
+            # use instantiateGlyphObject() to keep any custom sub-element classes
+            # https://github.com/googlefonts/ufo2ft/issues/363
+            g2 = g.layer.instantiateGlyphObject()
+            g2.name = name
+            return g2
 
     # copy everything except unused attributes: 'guidelines', 'note', 'image'
     glyphSet = obj_type()
@@ -98,6 +126,61 @@ def _copyLayer(layer, obj_type=dict):
     return glyphSet
 
 
+def deepCopyContours(
+    glyphSet, parent, composite, transformation, specificComponents=None
+):
+    """Copy contours from component to parent, including nested components.
+
+    specificComponent: an optional list of glyph name strings. If not passed or
+    None, decompose all components of a glyph unconditionally and completely. If
+    passed, only completely decompose components whose baseGlyph is in the list.
+    """
+
+    for nestedComponent in composite.components:
+        # Because this function works recursively, test at each turn if we are going to
+        # recurse into a specificComponent. If so, set the specificComponents argument
+        # to None so we unconditionally decompose the possibly nested component
+        # completely.
+        specificComponentsEffective = specificComponents
+        if specificComponentsEffective:
+            if nestedComponent.baseGlyph not in specificComponentsEffective:
+                continue
+            else:
+                specificComponentsEffective = None
+
+        try:
+            nestedBaseGlyph = glyphSet[nestedComponent.baseGlyph]
+        except KeyError:
+            logger.warning(
+                "dropping non-existent component '%s' in glyph '%s'",
+                nestedComponent.baseGlyph,
+                parent.name,
+            )
+        else:
+            deepCopyContours(
+                glyphSet,
+                parent,
+                nestedBaseGlyph,
+                transformation.transform(nestedComponent.transformation),
+                specificComponents=specificComponentsEffective,
+            )
+
+    # Check if there are any contours to copy before instantiating pens.
+    if composite != parent and len(composite):
+        if transformation == Identity:
+            pen = parent.getPen()
+        else:
+            pen = TransformPen(parent.getPen(), transformation)
+            # if the transformation has a negative determinant, it will
+            # reverse the contour direction of the component
+            xx, xy, yx, yy = transformation[:4]
+            if xx * yy - xy * yx < 0:
+                pen = ReverseContourPen(pen)
+
+        for contour in composite:
+            contour.draw(pen)
+
+
 def makeUnicodeToGlyphNameMapping(font, glyphOrder=None):
     """ Make a unicode: glyph name mapping for this glyph set (dict or Font).
 
@@ -204,76 +287,76 @@ def calcCodePageRanges(unicodes):
 
     for char in chars:
         if char == "Þ" and hasAscii:
-            codepageRanges.add(0)           # Latin 1
+            codepageRanges.add(0)  # Latin 1
         elif char == "Ľ" and hasAscii:
-            codepageRanges.add(1)           # Latin 2: Eastern Europe
+            codepageRanges.add(1)  # Latin 2: Eastern Europe
             if hasLineart:
-                codepageRanges.add(58)      # Latin 2
+                codepageRanges.add(58)  # Latin 2
         elif char == "Б":
-            codepageRanges.add(2)           # Cyrillic
+            codepageRanges.add(2)  # Cyrillic
             if "Ѕ" in chars and hasLineart:
-                codepageRanges.add(57)      # IBM Cyrillic
+                codepageRanges.add(57)  # IBM Cyrillic
             if "╜" in chars and hasLineart:
-                codepageRanges.add(49)      # MS-DOS Russian
+                codepageRanges.add(49)  # MS-DOS Russian
         elif char == "Ά":
-            codepageRanges.add(3)           # Greek
+            codepageRanges.add(3)  # Greek
             if hasLineart and "½" in chars:
-                codepageRanges.add(48)      # IBM Greek
+                codepageRanges.add(48)  # IBM Greek
             if hasLineart and "√" in chars:
-                codepageRanges.add(60)      # Greek, former 437 G
+                codepageRanges.add(60)  # Greek, former 437 G
         elif char == "İ" and hasAscii:
-            codepageRanges.add(4)           # Turkish
+            codepageRanges.add(4)  # Turkish
             if hasLineart:
-                codepageRanges.add(56)      # IBM turkish
+                codepageRanges.add(56)  # IBM turkish
         elif char == "א":
-            codepageRanges.add(5)           # Hebrew
+            codepageRanges.add(5)  # Hebrew
             if hasLineart and "√" in chars:
-                codepageRanges.add(53)      # Hebrew
+                codepageRanges.add(53)  # Hebrew
         elif char == "ر":
-            codepageRanges.add(6)           # Arabic
+            codepageRanges.add(6)  # Arabic
             if "√" in chars:
-                codepageRanges.add(51)      # Arabic
+                codepageRanges.add(51)  # Arabic
             if hasLineart:
-                codepageRanges.add(61)      # Arabic; ASMO 708
+                codepageRanges.add(61)  # Arabic; ASMO 708
         elif char == "ŗ" and hasAscii:
-            codepageRanges.add(7)           # Windows Baltic
+            codepageRanges.add(7)  # Windows Baltic
             if hasLineart:
-                codepageRanges.add(59)      # MS-DOS Baltic
+                codepageRanges.add(59)  # MS-DOS Baltic
         elif char == "₫" and hasAscii:
-            codepageRanges.add(8)           # Vietnamese
+            codepageRanges.add(8)  # Vietnamese
         elif char == "ๅ":
-            codepageRanges.add(16)          # Thai
+            codepageRanges.add(16)  # Thai
         elif char == "エ":
-            codepageRanges.add(17)          # JIS/Japan
+            codepageRanges.add(17)  # JIS/Japan
         elif char == "ㄅ":
-            codepageRanges.add(18)          # Chinese: Simplified chars
+            codepageRanges.add(18)  # Chinese: Simplified chars
         elif char == "ㄱ":
-            codepageRanges.add(19)          # Korean wansung
+            codepageRanges.add(19)  # Korean wansung
         elif char == "央":
-            codepageRanges.add(20)          # Chinese: Traditional chars
+            codepageRanges.add(20)  # Chinese: Traditional chars
         elif char == "곴":
-            codepageRanges.add(21)          # Korean Johab
+            codepageRanges.add(21)  # Korean Johab
         elif char == "♥" and hasAscii:
-            codepageRanges.add(30)          # OEM Character Set
+            codepageRanges.add(30)  # OEM Character Set
         # TODO: Symbol bit has a special meaning (check the spec), we need
         # to confirm if this is wanted by default.
         # elif unichr(0xF000) <= char <= unichr(0xF0FF):
         #    codepageRanges.add(31)          # Symbol Character Set
         elif char == "þ" and hasAscii and hasLineart:
-            codepageRanges.add(54)          # MS-DOS Icelandic
+            codepageRanges.add(54)  # MS-DOS Icelandic
         elif char == "╚" and hasAscii:
-            codepageRanges.add(62)          # WE/Latin 1
-            codepageRanges.add(63)          # US
+            codepageRanges.add(62)  # WE/Latin 1
+            codepageRanges.add(63)  # US
         elif hasAscii and hasLineart and "√" in chars:
             if char == "Å":
-                codepageRanges.add(50)      # MS-DOS Nordic
+                codepageRanges.add(50)  # MS-DOS Nordic
             elif char == "é":
-                codepageRanges.add(52)      # MS-DOS Canadian French
+                codepageRanges.add(52)  # MS-DOS Canadian French
             elif char == "õ":
-                codepageRanges.add(55)      # MS-DOS Portuguese
+                codepageRanges.add(55)  # MS-DOS Portuguese
 
     if hasAscii and "‰" in chars and "∑" in chars:
-        codepageRanges.add(29)              # Macintosh Character Set (US Roman)
+        codepageRanges.add(29)  # Macintosh Character Set (US Roman)
 
     # when no codepage ranges can be enabled, fall back to enabling bit 0
     # (Latin 1) so that the font works in MS Word:
@@ -285,7 +368,6 @@ def calcCodePageRanges(unicodes):
 
 
 class _LazyFontName(object):
-
     def __init__(self, font):
         self.font = font
 
@@ -293,3 +375,21 @@ class _LazyFontName(object):
         from ufo2ft.fontInfoData import getAttrWithFallback
 
         return getAttrWithFallback(self.font.info, "postscriptFontName")
+
+
+def getDefaultMasterFont(designSpaceDoc):
+    defaultSource = designSpaceDoc.findDefault()
+    if not defaultSource:
+        from ufo2ft.errors import InvalidDesignSpaceData
+
+        raise InvalidDesignSpaceData(
+            "Can't find base (neutral) master in DesignSpace document"
+        )
+    if not defaultSource.font:
+        from ufo2ft.errors import InvalidDesignSpaceData
+
+        raise InvalidDesignSpaceData(
+            "DesignSpace source '%s' is missing required 'font' attribute"
+            % getattr(defaultSource, "name", "<Unknown>")
+        )
+    return defaultSource.font
