@@ -1,12 +1,18 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function, absolute_import, division, unicode_literals
+from cu2qu.ufo import font_to_quadratic
 from fontTools.ttLib import TTFont
 from fontTools.misc.py23 import basestring, unichr, byteord
 from fontTools import designspaceLib
 from ufo2ft.outlineCompiler import OutlineTTFCompiler, OutlineOTFCompiler
 from ufo2ft.fontInfoData import intListToNum
 from fontTools.ttLib.tables._g_l_y_f import USE_MY_METRICS
-from ufo2ft.constants import USE_PRODUCTION_NAMES, GLYPHS_DONT_USE_PRODUCTION_NAMES
+from ufo2ft.constants import (
+    USE_PRODUCTION_NAMES,
+    GLYPHS_DONT_USE_PRODUCTION_NAMES,
+    SPARSE_TTF_MASTER_TABLES,
+    SPARSE_OTF_MASTER_TABLES,
+)
 from ufo2ft import (
     compileTTF,
     compileOTF,
@@ -32,59 +38,12 @@ def testufo(FontClass):
 
 
 @pytest.fixture
-def layertestrgufo(FontClass):
-    font = FontClass(getpath("LayerFont-Regular.ufo"))
+def quadufo(FontClass):
+    font = FontClass(getpath("TestFont.ufo"))
+    font_to_quadratic(font)
     return font
 
 
-@pytest.fixture
-def layertestbdufo(FontClass):
-    font = FontClass(getpath("LayerFont-Bold.ufo"))
-    return font
-
-
-@pytest.fixture
-def designspace(layertestrgufo, layertestbdufo):
-    ds = designspaceLib.DesignSpaceDocument()
-
-    a1 = designspaceLib.AxisDescriptor()
-    a1.tag = "wght"
-    a1.name = "Weight"
-    a1.default = a1.minimum = 350
-    a1.maximum = 625
-    ds.addAxis(a1)
-
-    s1 = designspaceLib.SourceDescriptor()
-    s1.name = "Layer Font Regular"
-    s1.familyName = "Layer Font"
-    s1.styleName = "Regular"
-    s1.filename = "LayerFont-Regular.ufo"
-    s1.location = {"Weight": 350}
-    s1.font = layertestrgufo
-    ds.addSource(s1)
-
-    s2 = designspaceLib.SourceDescriptor()
-    s2.name = "Layer Font Medium"
-    s2.familyName = "Layer Font"
-    s2.styleName = "Medium"
-    s2.filename = "LayerFont-Regular.ufo"
-    s2.layerName = "Medium"
-    s2.location = {"Weight": 450}
-    s2.font = layertestrgufo
-    ds.addSource(s2)
-
-    s3 = designspaceLib.SourceDescriptor()
-    s3.name = "Layer Font Bold"
-    s3.familyName = "Layer Font"
-    s3.styleName = "Bold"
-    s3.filename = "LayerFont-Bold.ufo"
-    s3.location = {"Weight": 625}
-    s3.font = layertestbdufo
-    ds.addSource(s3)
-
-    return ds
-
-
 @pytest.fixture
 def use_my_metrics_ufo(FontClass):
     return FontClass(getpath("UseMyMetrics.ufo"))
@@ -130,9 +89,8 @@ class OutlineTTFCompilerTest(object):
         compiler.compile()
         assert "gasp" not in compiler.otf
 
-    def test_makeGlyphsBoundingBoxes(self, testufo):
-        # the call to 'makeGlyphsBoundingBoxes' happen in the __init__ method
-        compiler = OutlineTTFCompiler(testufo)
+    def test_makeGlyphsBoundingBoxes(self, quadufo):
+        compiler = OutlineTTFCompiler(quadufo)
         assert compiler.glyphBoundingBoxes[".notdef"] == (50, 0, 450, 750)
         # no outline data
         assert compiler.glyphBoundingBoxes["space"] is None
@@ -447,17 +405,7 @@ class OutlineOTFCompilerTest(object):
 
         self.assertProgramEqual(
             program,
-            [
-                -12,
-                66,
-                'hmoveto',
-                256,
-                'hlineto',
-                -128,
-                510,
-                'rlineto',
-                'endchar'
-            ]
+            [-12, 66, "hmoveto", 256, "hlineto", -128, 510, "rlineto", "endchar"],
         )
 
     def test_setupTable_CFF_no_optimize(self, testufo):
@@ -469,23 +417,10 @@ class OutlineOTFCompilerTest(object):
 
         self.assertProgramEqual(
             program,
-            [
-                -12,
-                66,
-                0,
-                'rmoveto',
-                256,
-                0,
-                'rlineto',
-                -128,
-                510,
-                'rlineto',
-                'endchar'
-            ],
+            [-12, 66, 0, "rmoveto", 256, 0, "rlineto", -128, 510, "rlineto", "endchar"],
         )
 
     def test_makeGlyphsBoundingBoxes(self, testufo):
-        # the call to 'makeGlyphsBoundingBoxes' happen in the __init__ method
         compiler = OutlineOTFCompiler(testufo)
         # with default roundTolerance, all coordinates and hence the bounding
         # box values are rounded with otRound()
@@ -559,6 +494,31 @@ class OutlineOTFCompilerTest(object):
         # 250 - nominalWidthX
         assert charStrings.getItemAndSelector("space")[0].program == [-53, "endchar"]
 
+    def test_optimized_default_but_no_nominal_widths(self, FontClass):
+        ufo = FontClass()
+        ufo.info.familyName = "Test"
+        ufo.info.styleName = "R"
+        ufo.info.ascender = 1
+        ufo.info.descender = 1
+        ufo.info.capHeight = 1
+        ufo.info.xHeight = 1
+        ufo.info.unitsPerEm = 1000
+        ufo.info.postscriptDefaultWidthX = 500
+        for glyphName, width in (
+            (".notdef", 500),
+            ("space", 500),
+            ("a", 500),
+        ):
+            glyph = ufo.newGlyph(glyphName)
+            glyph.width = width
+
+        font = compileOTF(ufo)
+        cff = font["CFF "].cff
+        private = cff[list(cff.keys())[0]].Private
+
+        assert private.defaultWidthX == 500
+        assert private.nominalWidthX == 0
+
 
 class GlyphOrderTest(object):
     def test_compile_original_glyph_order(self, testufo):
@@ -810,6 +770,33 @@ class NamesTest(object):
         assert name in result
 
 
+class ColrCpalTest:
+
+    def test_colr_cpal(self, FontClass):
+        testufo = FontClass(getpath("ColorTest.ufo"))
+        assert "com.github.googlei18n.ufo2ft.colorLayerMapping" in testufo.lib
+        assert "com.github.googlei18n.ufo2ft.colorPalettes" in testufo.lib
+        result = compileTTF(testufo)
+        assert "COLR" in result
+        assert "CPAL" in result
+        layers = {gn: [(layer.name, layer.colorID) for layer in layers]
+                      for gn, layers in result["COLR"].ColorLayers.items()}
+        assert layers == {'a': [('a.color1', 0), ('a.color2', 1)],
+                          'b': [('b.color1', 1), ('b.color2', 0)],
+                          'c': [('c.color2', 1), ('c.color1', 0)]}
+
+    def test_colr_cpal_raw(self, FontClass):
+        testufo = FontClass(getpath("ColorTestRaw.ufo"))
+        assert "com.github.googlei18n.ufo2ft.colorLayers" in testufo.lib
+        assert "com.github.googlei18n.ufo2ft.colorPalettes" in testufo.lib
+        result = compileTTF(testufo)
+        palettes = [[(c.red, c.green, c.blue, c.alpha) for c in p] for p in result["CPAL"].palettes]
+        assert palettes == [[(255, 76, 26, 255), (0, 102, 204, 255)]]
+        layers = {gn: [(layer.name, layer.colorID) for layer in layers]
+                      for gn, layers in result["COLR"].ColorLayers.items()}
+        assert layers == {"a": [('a.color1', 0), ('a.color2', 1)]}
+
+
 ASCII = [unichr(c) for c in range(0x20, 0x7E)]
 
 
@@ -881,10 +868,7 @@ def test_custom_layer_compilation_interpolatable(layertestrgufo, layertestbdufo)
     ufo2 = layertestbdufo
 
     master_ttfs = list(
-        compileInterpolatableTTFs(
-            [ufo1, ufo1, ufo2],
-            layerNames=[None, "Medium", None],
-        )
+        compileInterpolatableTTFs([ufo1, ufo1, ufo2], layerNames=[None, "Medium", None])
     )
     assert master_ttfs[0].getGlyphOrder() == [
         ".notdef",
@@ -904,6 +888,9 @@ def test_custom_layer_compilation_interpolatable(layertestrgufo, layertestbdufo)
         "edotabove",
     ]
 
+    sparse_tables = [tag for tag in master_ttfs[1].keys() if tag != "GlyphOrder"]
+    assert SPARSE_TTF_MASTER_TABLES.issuperset(sparse_tables)
+
 
 @pytest.mark.parametrize("inplace", [False, True], ids=["not inplace", "inplace"])
 def test_custom_layer_compilation_interpolatable_from_ds(designspace, inplace):
@@ -930,6 +917,14 @@ def test_custom_layer_compilation_interpolatable_from_ds(designspace, inplace):
         "edotabove",
     ]
 
+    sparse_tables = [tag for tag in master_ttfs[1].keys() if tag != "GlyphOrder"]
+    assert SPARSE_TTF_MASTER_TABLES.issuperset(sparse_tables)
+
+    # sentinel value used by varLib to ignore the post table for this sparse
+    # master when building the MVAR table
+    assert master_ttfs[1]["post"].underlinePosition == -0x8000
+    assert master_ttfs[1]["post"].underlineThickness == -0x8000
+
 
 @pytest.mark.parametrize("inplace", [False, True], ids=["not inplace", "inplace"])
 def test_custom_layer_compilation_interpolatable_otf_from_ds(designspace, inplace):
@@ -956,6 +951,9 @@ def test_custom_layer_compilation_interpolatable_otf_from_ds(designspace, inplac
         "edotabove",
     ]
 
+    sparse_tables = [tag for tag in master_otfs[1].keys() if tag != "GlyphOrder"]
+    assert SPARSE_OTF_MASTER_TABLES.issuperset(sparse_tables)
+
 
 def test_compilation_from_ds_missing_source_font(designspace):
     designspace.sources[0].font = None
@@ -963,6 +961,19 @@ def test_compilation_from_ds_missing_source_font(designspace):
         compileInterpolatableTTFsFromDS(designspace)
 
 
+def test_compile_empty_ufo(FontClass):
+    ufo = FontClass()
+    font = compileTTF(ufo)
+    assert font["name"].getName(1, 3, 1).toUnicode() == "New Font"
+    assert font["name"].getName(2, 3, 1).toUnicode() == "Regular"
+    assert font["name"].getName(4, 3, 1).toUnicode() == "New Font Regular"
+    assert font["head"].unitsPerEm == 1000
+    assert font["OS/2"].sTypoAscender == 800
+    assert font["OS/2"].sCapHeight == 700
+    assert font["OS/2"].sxHeight == 500
+    assert font["OS/2"].sTypoDescender == -200
+
+
 if __name__ == "__main__":
     import sys
 
